@@ -6,25 +6,29 @@
 ## Package <main>
 ##############################
 __all__ = """
+    totuple
+    recursive_apply
     vector
     generator_wrapper
     ctgenerator
 """.split()
 
-from typing import Optional, List, Union
 from types import GeneratorType
 from collections import Counter
 from pyoverload import *
 from functools import wraps
+from .touch import touch, crash
 
 """
 Usage:
 from pyctlib.vector import *
 """
 
-def touch(f: Callable):
-    try: return f()
-    except: return None
+def totuple(x, l=None):
+    if not isinstance(x, tuple): x = (x,)
+    if l is None: l = len(x)
+    if l % len(x) == 0: return x * (l // len(x))
+    raise TypeError(f"{x} can not be converted into a tuple of length {l}. ")
 
 def raw_function(func):
     if "__func__" in dir(func):
@@ -39,6 +43,22 @@ class _Vector_Dict(dict):
     def keys(self):
         return vector(super().keys())
 
+def recursive_apply(container, func):
+    if isinstance(container, vector):
+        return container.map(lambda x: recursive_apply(x, func))
+    if isinstance(container, list):
+        return [recursive_apply(x, func) for x in container]
+    if isinstance(container, tuple):
+        return tuple([recursive_apply(x, func) for x in container])
+    if isinstance(container, set):
+        return set([recursive_apply(x, func) for x in container])
+    if isinstance(container, dict):
+        return {key: recursive_apply(value, func) for key, value in enumerate(container)}
+    try:
+        return func(container)
+    except:
+        return container
+
 class vector(list):
 
     def __init__(self, *args):
@@ -49,16 +69,22 @@ class vector(list):
         else:
             list.__init__(self, args)
 
-    def filter(self, func=None):
+    def filter(self, func=None, ignore_error=True):
         if func is None:
             return self
         try:
+            if ignore_error:
+                return vector([a for a in self if touch(lambda: func(a))])
             return vector([a for a in self if func(a)])
         except:
             pass
         for index, a in enumerate(self):
             if touch(lambda: func(a)) is None:
-                raise RuntimeError("Exception raised in filter function at location {} for element {}".format(index, a))
+                try:
+                    error_information = "Exception raised in filter function at location {} for element {}".format(index, a)
+                except:
+                    error_information = "Exception raised in filter function at location {} for element {}".format(index, "<unknown>")
+                raise RuntimeError(error_information)
 
     def test(self, func):
         return vector([a for a in self if touch(lambda: func(a))])
@@ -78,12 +104,22 @@ class vector(list):
             pass
         for index, a in enumerate(self):
             if touch(lambda: func(a)) is None:
-                raise RuntimeError("Exception raised in map function at location {} for element {}".format(index, a))
+                try:
+                    error_information = "Exception raised in map function at location {} for element {}".format(index, a)
+                except:
+                    error_information = "Exception raised in map function at location {} for element {}".format(index, "<unknown>")
+                raise RuntimeError(error_information)
 
-    def apply(self, func) -> None:
+    @overload
+    def apply(self, func: Functional) -> None:
         for x in self:
             func(x)
 
+    @overload
+    def apply(self, command: str) -> None:
+        for x in self:
+            exec(command.format(x))
+
     def check_type(self, instance):
         return all(self.map(lambda x: isinstance(x, instance)))
 
@@ -161,17 +197,14 @@ class vector(list):
     def _(self, other: list):
         return vector(zip(self, other)).map(lambda x: x[0] >= x[1])
 
-    @override
     def __getitem__(self, index):
         if isinstance(index, slice):
             return vector(super().__getitem__(index))
+        if isinstance(index, list):
+            assert len(self) == len(index)
+            return vector(zip(self, index)).filter(lambda x: x[1]).map(lambda x: x[0])
         return super().__getitem__(index)
 
-    @__getitem__
-    def _(self, index_list: list):
-        assert len(self) == len(index_list)
-        return vector(zip(self, index_list)).filter(lambda x: x[1]).map(lambda x: x[0])
-
     @overload
     def __sub__(self, other: Iterable):
         try:
@@ -238,36 +271,50 @@ class vector(list):
                 pushfunc(x)
         return vector(unique_elements)
 
+    def count_all(self):
+        if len(self) == 0:
+            return vector([])
+        hashable = self._hashable()
+        if hashable:
+            return Counter(self)
+        else:
+            return self.unique().map(lambda x: (x, self.count(x)))
+
     def count(self, *args):
         if len(args) == 0:
             return len(self)
         return super().count(args[0])
 
-    # @overload
-    # def index(self, element: int):
-    #     return super().index(element)
-
-    # @overload
     def index(self, element):
-        if isinstance(element, int):
-            return super().index(element)
-        elif callable(element):
+        # if isinstance(element, int):
+        #     return super().index(element)
+        if callable(element):
             for index in range(len(self)):
-                if element(self[index]):
+                if touch(lambda: element(self[index])):
                     return index
             return -1
         else:
-            raise RuntimeError("error input for index")
+            return super().index(element)
+
+    def findall(self, element):
+        if callable(element):
+            return vector([index for index in range(len(self)) if touch(lambda: element(self[index]))])
+        else:
+            return vector([index for index in range(len(self)) if self[index] == element])
+
+    def findall_crash(self, func):
+        assert callable(func)
+        return vector([index for index in range(len(self)) if crash(lambda: func(self[index]))])
 
     def all(self, func=lambda x: x):
         for t in self:
-            if not func(t):
+            if not touch(lambda: func(t)):
                 return False
         return True
 
     def any(self, func=lambda x: x):
         for t in self:
-            if func(t):
+            if touch(lambda: func(t)):
                 return True
         return False
 
@@ -303,6 +350,9 @@ class vector(list):
     def sum(self):
         return self.reduce(lambda x, y: x + y)
 
+    def prod(self):
+        return self.reduce(lambda x, y: x * y)
+
     def group_by(self, key=lambda x: x[0]):
         result = _Vector_Dict()
         for x in self:
@@ -327,6 +377,22 @@ class vector(list):
     def generator(self):
         return ctgenerator(self)
 
+def recursive_apply(container, func):
+    if isinstance(container, vector):
+        return container.map(lambda x: recursive_apply(x, func))
+    if isinstance(container, list):
+        return [recursive_apply(x, func) for x in container]
+    if isinstance(container, tuple):
+        return tuple([recursive_apply(x, func) for x in container])
+    if isinstance(container, set):
+        return set([recursive_apply(x, func) for x in container])
+    if isinstance(container, dict):
+        return {key: recursive_apply(value, func) for key, value in enumerate(container)}
+    try:
+        return func(container)
+    except:
+        return container
+
 def generator_wrapper(*args, **kwargs):
     if len(args) == 1 and callable(raw_function(args[0])):
         func = raw_function(args[0])
